// Copyright (C) 2020 Intel Corporation

// SPDX-License-Identifier: MIT

#include <CL/sycl.hpp>
#include <iostream>
#include <CL/sycl/INTEL/fpga_extensions.hpp> // For fpga_selector
using namespace sycl;

void show_device_info (const char *nm, device d){

  std::cout << "\n FOR:" << nm << "\n";
  std::cout << "Selected device: " <<
  d.get_info<info::device::name>() << "\n";

   
   std::cout << "host:" << d.has(aspect::host) << "\n";
   std::cout << "cpu:" << d.has(aspect::cpu) << "\n";
   std::cout << "gpu:" << d.has(aspect::gpu) << "\n";
   std::cout << "accelerator:" << d.has(aspect::accelerator) << "\n";
   std::cout << "custom:" << d.has(aspect::custom) << "\n";
   std::cout << "fp16:" << d.has(aspect::fp16) << "\n";
   std::cout << "fp64:" << d.has(aspect::fp64) << "\n";
   std::cout << "int64_base_atomics:" << d.has(aspect::int64_base_atomics) << "\n";
   std::cout << "int64_extended_atomics:" << d.has(aspect::int64_extended_atomics) << "\n";
   std::cout << "image:" << d.has(aspect::image) << "\n";
   std::cout << "online_compiler:" << d.has(aspect::online_compiler) << "\n";
   std::cout << "queue_profiling:" << d.has(aspect::queue_profiling) << "\n";
   std::cout << "usm_device_allocations:" << d.has(aspect::usm_device_allocations) << "\n";
   std::cout << "usm_host_allocations:" << d.has(aspect::usm_host_allocations) << "\n";
   std::cout << "usm_shared_allocations:" << d.has(aspect::usm_shared_allocations) << "\n";
   std::cout << "usm_restricted_shared_allocations:" << d.has(aspect::usm_restricted_shared_allocations) << "\n";
   std::cout << "usm_system_allocator:" << d.has(aspect::usm_system_allocator) << "\n";
   std::cout << "ext_intel_pci_address:" << d.has(aspect::ext_intel_pci_address) << "\n";
   std::cout << "ext_intel_gpu_eu_count:" << d.has(aspect::ext_intel_gpu_eu_count) << "\n";
   std::cout << "ext_intel_gpu_eu_simd_width:" << d.has(aspect::ext_intel_gpu_eu_simd_width) << "\n";
   std::cout << "ext_intel_gpu_slices:" << d.has(aspect::ext_intel_gpu_slices) << "\n";
   std::cout << "ext_intel_gpu_eu_count_per_subslice:" << d.has(aspect::ext_intel_gpu_eu_count_per_subslice) << "\n";
   std::cout << "ext_intel_max_mem_bandwidth:" << d.has(aspect::ext_intel_max_mem_bandwidth) << "\n";
   //not implemanted std::cout << "ext_intel_mem_channel:" << d.has(aspect::ext_intel_mem_channel) << "\n";

  std::cout << "is_cpu:" << d.is_cpu() << "\n";
  std::cout << "is_gpu:" << d.is_gpu() << "\n";
  std::cout << "is_host:" << d.is_host() << "\n";
  std::cout << "is_accelerator:" << d.is_accelerator() << "\n";
}

int main() {
  // Create queue on whatever default device that the implementation
  // chooses. Implicit use of the default_selector. 
  queue Q;
  // default device:
  device d = Q.get_device();
  show_device_info("default device",d);
  d=device(host_selector());
  show_device_info("host_selector", d);
  d=device(gpu_selector());
  show_device_info("GPU_selector", d);
  d=device(INTEL::fpga_emulator_selector());
  show_device_info("INTEL::fpga_emulator_selector", d);
  d=device{gpu_selector{}};
  show_device_info("GPU_selector{}", d);


  return 0;
}

